/*********************************************************************
 * teleinfo.c      -  EDF Teleinfo data receptor
 * Dependencies:    
 * Processor:       PIC18
 * Complier:        MCC18 v1.00.50 or higher
 *  
 *
 *********************************************************************
 * Description
 *
 * Allows the SBC65EC to receive EDF releinformation serial data
 *
 *********************************************************************
 * File History
 *
 * 2009-06-16, Benot Sandr (BS):
 *    - Initial Version
 *
 *********************************************************************
 * Software License Agreement : according GPL 2.0 
 *
 ********************************************************************/


#include "projdefs.h"
#include "serint.h"
#include "net\tick.h"
#include "teleinfo.h"

#pragma udata buffer1
BYTE trameTeleInfo1[TAILLE_MAX_TRAME_TELEINFO];  // buffer used to save teleinfo frame
#pragma udata buffer2
BYTE trameTeleInfo2[TAILLE_MAX_TRAME_TELEINFO];  // buffer used to save teleinfo frame
#pragma udata
BYTE *teleinfoBuffer; //pointer on one of above buffer


BYTE statusTeleInfo;  			// variable used to store teleinfo receive status
TICK16 startTimer;  							// Variable used to chek RX timeout


//function used to run teleinfo tasks
BYTE processTeleInfo() {
static BYTE indexTrame;
BYTE RxByte;
	if (statusTeleInfo==TELEINFO_POWER_ON)  {

		//if power is on since sufficient time
		if (TickGetDiff16bit(startTimer) > TELEINFO_POWER_ON_TIMER) {
			//clear serial buffer
			serReset();
			//change state of task
			statusTeleInfo=TELEINFO_START_WAITING;
			startTimer=TickGet16bit();
		}
	}
	else if (statusTeleInfo==TELEINFO_START_WAITING)  {

		//timeout check
		if (TickGetDiff16bit(startTimer) > TELEINFO_TIMEOUT) {
			teleinfoBuffer[0]=0;
			POWER_OFF_TI;
			statusTeleInfo=TELEINFO_RX_KO;
		}
		//parse received data
		else while (serIsGetReady() && (statusTeleInfo==TELEINFO_START_WAITING)) {
			RxByte=serGetByte() & 0x7F;
			//if a frame start has been deteted
			if (RxByte ==2) {
				indexTrame=0;
				statusTeleInfo=TELEINFO_RECEIVING;
				startTimer=TickGet16bit();
			} 
		}
	}
	else if (statusTeleInfo==TELEINFO_RECEIVING) {
		//traitement du timeout
		if (TickGetDiff16bit(startTimer) > TELEINFO_TIMEOUT) {
			teleinfoBuffer[0]=0;
			POWER_OFF_TI;
			statusTeleInfo=TELEINFO_RX_KO;
		}
		//On recupere les infos du compteur
		 else while (serIsGetReady() && (statusTeleInfo==TELEINFO_RECEIVING)) {
			RxByte=serGetByte() & 0x7F;
			if (( RxByte == 3 ) || ( RxByte == 4 )) {
				teleinfoBuffer[indexTrame++]=0;
				POWER_OFF_TI;
				statusTeleInfo=TELEINFO_RX_OK;
			} else {
				if ( indexTrame !=TAILLE_MAX_TRAME_TELEINFO-2) teleinfoBuffer[indexTrame++]=RxByte; 
			}
		}
	}
	return(statusTeleInfo);
}

