/**
 * @brief           leds management
 * @file            websampling.c
 * @author          <a href="">Dom-IP</a>
 * @dependencies    none
 * @compiler        MPLAB C18 v2.10 or higher <br>
 *                  HITECH PICC-18 V8.35PL3 or higher
 *
 * @section description Description
 **********************************
 * This module contains functions used for the leds management of evaluation boards
 *
 *********************************************************************/

 /*********************************************************************
 * Software License Agreement
 *
 * The software supplied herewith is owned by Benoit Sandre, and is
 * protected under applicable copyright laws.  The code may
 * be modified and can be used free of charge for commercial and non commercial
 * applications. All rights are reserved. Any use in violation of the foregoing
 * restrictions may subject the user to criminal sanctions under applicable laws,
 * as well as to civil liability for the breach of the terms and conditions of this license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN 'AS IS' CONDITION. NO WARRANTIES, WHETHER EXPRESS,
 * IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE
 * COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 **********************************************************************
 * File History
 *
 * 2009-06-01, Dom-IP
 *    - Initial version
 *********************************************************************/
#include <string.h>
#include "projdefs.h"
#include "appcfg.h"
#include "net\tick.h"
#include "net\compiler.h"
#include "leds.h"
#include "net\dns.h"
#include "net\httpclient.h"
#include "teleinfo.h"
#include "websampling.h"

/////////////////////////////////////////////////
//Function prototypes
//NA

////////////////////////////////
//Websampling application variables
////////////////////////////////
BYTE Status=0;
TICK16 tmrSec;
unsigned short nameResolutionCount=0;


//WebSampling application initialization
void appcfgWebSampling() {
	//Init Teleinfo
	INIT_TELEINFO;
	//init Timer
	tmrSec=TickGet16bitSec()-((TICK16) WS_INTERVAL );
	//force  dns request
	nameResolutionCount=0;
	//launch task if no http request in progress
	if (Status!=8) Status=0;
}

//WebSampling application processing
void appWebSampling() {
static BYTE HostName[32];
static BYTE filename[9];
static IP_ADDR HostIP;
static TCP_FILESEND_INFO Req_Http;
BYTE i;
		//In standby state
		if (Status==0) {
		//Every 60 seconds, 
			SetLeds(STANDBY_LED);
			if (  TickGetDiffSec(tmrSec) >= ((TICK16) WS_INTERVAL ) ) {
				//reload the seconds timer
				tmrSec+=((TICK16)60);
				//Relaunch send if task is in Standby
				Status=1;
				ClearLeds(STANDBY_LED);
			}
		}
		// DNS Resolution init
		else if (Status==1) {
			// Si la rsolution DNS doit tre lance
			if (nameResolutionCount--==0) {
				//Reinit Tmr
				nameResolutionCount=NAME_RESOLUTION_MAX_COUNT;
				
				//Get host name from EEPROM
				for(i=0;i<32;i++) {
					HostName[i]=appcfgGetc(APPCFG_HOST_NAME0+i);
				}
				//string max length is 32 chars
				HostName[31]=0;
				
				// If hostname is defined
				if (HostName[0]!=0) {
					//Launch DNS request
					DNSResolve(HostName);
					//Status update
					SetLeds(DNS_LED);
					Status=2;
				}
				// Otherwise launch teleinfo frame receiving 
				else Status=3;
			}
			// Otherwise launch teleinfo 1 frame receiving 
			else Status=3;
		}
		//Waiting for DNS resolution
		else if (Status==2) {
			//if dns request is ended
			if ( DNSIsResolved(&HostIP)) {
				//Save result in EEPROM
				for(i=0;i<4;i++) {
					appcfgPutc(APPCFG_HOST_IP0+i, HostIP.v[i]);
				}
				//Launch teleinfo 1 frame receiving 
				ClearLeds(DNS_LED);
				Status=3;
			}
		}
		//init teleinformation 1  frame reception
		else if (Status==3)  {
			SetLeds(TELEINFO1_LED);
			//start teleinfo frame 1 reception
			START_TELEINFO_RX1;
			Status=4;
		}
		//teleinformation frame reception
		else if (Status==4) {
			//if teleinfo reception is ended, launch sending of http request
			i=processTeleInfo();
			//if frame has been successfully received
			if (i==TELEINFO_RX_OK) {
				//switch led off
				ClearLeds(TELEINFO1_LED);
				Status=5;
			}
			//if an error occur led stay on
			if (i==TELEINFO_RX_KO) Status=5;
		}
		//init teleinformation 2  frame reception
		else if (Status==5)  {
			SetLeds(TELEINFO2_LED);
			//start teleinfo frame 2 reception
			START_TELEINFO_RX2;
			Status=6;
		}
		//teleinformation 2 frame reception
		else if (Status==6) {
			//if teleinfo reception is ended, launch sending of http request
			i=processTeleInfo();
			//if frame has been successfully received
			if (i==TELEINFO_RX_OK) {
				//switch led off
				ClearLeds(TELEINFO2_LED);			
				Status=7;
			}
			//if an error occur led stay on
			if (i==TELEINFO_RX_KO) Status=7;
		}
		//Init HTTP request
		else if (Status==7) {
			SetLeds(HTTP_LED);
			//Initialize http request
			Req_Http.port=HTTP_SERVICE_PORT;
			Req_Http.serverNode.IPAddr.v[0]=appcfgGetc(APPCFG_HOST_IP0);
			Req_Http.serverNode.IPAddr.v[1]=appcfgGetc(APPCFG_HOST_IP1);
			Req_Http.serverNode.IPAddr.v[2]=appcfgGetc(APPCFG_HOST_IP2);
			Req_Http.serverNode.IPAddr.v[3]=appcfgGetc(APPCFG_HOST_IP3);
			Req_Http.sm=0;
			
			//The request data is saved in the file "requete.htp"
			strcpypgm2ram(filename,"httpreq.cgi");
			Status=8;
		}
		//Continue HTTP request
		else if (Status==8) {
			//start or continue http request
			i=sendFileToTcpServer(filename,&Req_Http);
			//http request has been send successfully
			if (i==0) {
				//switch led off
				ClearLeds(HTTP_LED);			
				Status=0;
			}
			//if an error occur, led stay on
			else if (i!=0xff) Status=0;
		}

}

BYTE WS_State() {
	return(Status);
}



